==> doot-0.3.0-py3-none-any.whl: doot/task/base_tasker.py <==
#!/usr/bin/env python3
"""
Utility classes for building tasks with a bit of structure
"""
##-- imports
from __future__ import annotations

# import abc
# import datetime
# import enum
import functools as ftz
import itertools as itz
import logging as logmod
import pathlib as pl
import re
import time
import types
# from copy import deepcopy
# from dataclasses import InitVar, dataclass, field
from typing import (TYPE_CHECKING, Any, Callable, ClassVar, Final, Generic,
                    Iterable, Iterator, Mapping, Match, MutableMapping,
                    Protocol, Sequence, Tuple, TypeAlias, TypeGuard, TypeVar,
                    cast, final, overload, runtime_checkable, Generator)
# from uuid import UUID, uuid1
# from weakref import ref

##-- end imports

##-- logging
logging = logmod.getLogger(__name__)
##-- end logging

from tomlguard import TomlGuard
import doot
import doot.errors
from doot.constants import SUBTASKED_HEAD
from doot.enums import TaskFlags
from doot.structs import DootTaskSpec, TaskStub, TaskStubPart, DootTaskName
from doot._abstract import Tasker_i, Task_i
from doot.mixins.importer import ImporterMixin
from doot.errors import DootDirAbsent

@doot.check_protocol
class DootTasker(Tasker_i, ImporterMixin):
    """ Util Class for building single tasks
      wraps with setup and teardown tasks,
      manages cleaning,
      and holds state

    """
    _help = ["A Basic Task Constructor"]
    _default_flags = TaskFlags.TASKER

    def __init__(self, spec:DootTaskSpec):
        assert(spec is not None), "Spec is empty"
        super(DootTasker, self).__init__(spec)

    def default_task(self, name:str|DootTaskName|None, extra:None|dict|TomlGuard) -> DootTaskSpec:
        task_name = None
        match name:
            case None:
                task_name = self.fullname.subtask(SUBTASKED_HEAD)
            case str():
                task_name = self.fullname.subtask(name)
            case DootTaskName():
                task_name = name
            case _:
                raise doot.errors.DootTaskError("Bad value used to make a subtask in %s : %s", self.name, name)

        assert(task_name is not None)
        return DootTaskSpec(name=task_name, extra=TomlGuard(extra))

    def is_stale(self, task:Task_i):
        return False

    def build(self, **kwargs) -> Generator[DootTaskSpec]:
        logging.debug("-- tasker %s expanding tasks", self.name)
        if bool(kwargs):
            logging.debug("received kwargs: %s", kwargs)
            self.args.update(kwargs)

        yield self._build_head()

    def specialize_task(self, task):
        return task

    @classmethod
    def stub_class(cls, stub) -> TaskStub:
        stub.ctor                 = cls
        stub['version'].default   = cls._version
        stub['doc'].default       = [f"\"{x}\"" for x  in cls.class_help().split("\n")]
        stub['flags'].default     = cls._default_flags
        stub['flags'].prefix      = "# "
        stub['head_task'].type    = "task_iden"
        stub['head_task'].default = ""
        stub['head_task'].prefix  = "# "
        stub['queue_behaviour'].default = "default"
        stub['queue_behaviour'].comment = "default | auto | reactive"
        return stub

    def stub_instance(self, stub) -> TaskStub:
        stub                      = self.__class__.stub_class(stub)
        stub['name'].default      = self.fullname
        if bool(self.doc):
            stub['doc'].default   = [f"\"{x}\"" for x in self.doc]
        stub['flags'].default     = self.spec.flags
        return stub

    def _build_head(self, **kwargs) -> DootTaskSpec:
        logging.debug("Building Head Task for: %s", self.name)
        task_spec                             = self.default_task(None, TomlGuard(kwargs))

        task_ref = self.spec.extra.on_fail((None,), None|str).head_task()
        if task_ref is not None:
            task_spec.ctor_name = DootTaskName.from_str(task_ref)

        maybe_task : DootTaskSpec | None = self.specialize_task(task_spec)

        match maybe_task:
            case None:
                raise DootTaskError("Task Failed to specialize the head task: %s", self.name)
            case _ if not bool(maybe_task.doc):
                maybe_task.doc = self.doc
                return maybe_task
            case _:
                return maybe_task

==> doot-0.3.1-py3-none-any.whl: doot/task/base_tasker.py <==
#!/usr/bin/env python3
"""
Utility classes for building tasks with a bit of structure
"""
##-- imports
from __future__ import annotations

# import abc
# import datetime
# import enum
import functools as ftz
import itertools as itz
import logging as logmod
import pathlib as pl
import re
import time
import types
# from copy import deepcopy
# from dataclasses import InitVar, dataclass, field
from typing import (TYPE_CHECKING, Any, Callable, ClassVar, Final, Generic,
                    Iterable, Iterator, Mapping, Match, MutableMapping,
                    Protocol, Sequence, Tuple, TypeAlias, TypeGuard, TypeVar,
                    cast, final, overload, runtime_checkable, Generator)
# from uuid import UUID, uuid1
# from weakref import ref

##-- end imports

##-- logging
logging = logmod.getLogger(__name__)
##-- end logging

from tomlguard import TomlGuard
import doot
import doot.errors
from doot.constants import SUBTASKED_HEAD
from doot.enums import TaskFlags
from doot.structs import DootTaskSpec, TaskStub, TaskStubPart, DootTaskName, DootCodeReference, DootStructuredName
from doot._abstract import Tasker_i, Task_i
from doot.mixins.importer import ImporterMixin
from doot.errors import DootDirAbsent

@doot.check_protocol
class DootTasker(Tasker_i, ImporterMixin):
    """ Util Class for building single tasks
      wraps with setup and teardown tasks,
      manages cleaning,
      and holds state

    """
    _help = ["A Basic Task Constructor"]
    _default_flags = TaskFlags.TASKER

    def __init__(self, spec:DootTaskSpec):
        assert(spec is not None), "Spec is empty"
        super(DootTasker, self).__init__(spec)

    def default_task(self, name:str|DootTaskName|None, extra:None|dict|TomlGuard) -> DootTaskSpec:
        task_name = None
        match name:
            case None:
                task_name = self.fullname.subtask(SUBTASKED_HEAD)
            case str():
                task_name = self.fullname.subtask(name)
            case DootTaskName():
                task_name = name
            case _:
                raise doot.errors.DootTaskError("Bad value used to make a subtask in %s : %s", self.name, name)

        assert(task_name is not None)
        return DootTaskSpec(name=task_name, extra=TomlGuard(extra))

    def is_stale(self, task:Task_i):
        return False

    def build(self, **kwargs) -> Generator[DootTaskSpec]:
        logging.debug("-- tasker %s expanding tasks", self.name)
        if bool(kwargs):
            logging.debug("received kwargs: %s", kwargs)
            self.args.update(kwargs)

        head = self._build_head()
        yield self.specialize_task(head)

    def specialize_task(self, task):
        return task

    @classmethod
    def stub_class(cls, stub) -> TaskStub:
        stub.ctor                 = cls

        # Come first
        stub['active_when'].priority    = -90
        stub['required_for'].priority   = -90
        stub['depends_on'].priority     = -100

        stub['head_task'].set(type="taskname", default="", prefix="# ", priority=100)
        stub['queue_behaviour'].default = "default"
        stub['queue_behaviour'].comment = "default | auto | reactive"
        return stub

    def stub_instance(self, stub) -> TaskStub:
        stub                      = self.__class__.stub_class(stub)
        stub['name'].default      = self.fullname
        if bool(self.doc):
            stub['doc'].default   = [f"\"{x}\"" for x in self.doc]
        return stub

    def _build_head(self, **kwargs) -> None|DootTaskSpec:
        logging.debug("Building Head Task for: %s", self.name)
        task_spec = self.default_task(None, TomlGuard(kwargs))

        task_ref = self.spec.extra.on_fail((None,), None|str).head_task()
        if task_ref is not None:
            task_spec.ctor = DootStructuredName.build(task_ref)

        return task_spec
